@@ -1,24 +1,14 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 from typing import cast
 from typing import NamedTuple
-from typing import Optional
 
+from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import Gtk
 from gi.repository import Pango
@@ -41,7 +31,7 @@ class DialogButton(NamedTuple):
     is_default: bool
 
     @classmethod
-    def make(cls, type_: Optional[str] = None, **kwargs: Any) -> DialogButton:
+    def make(cls, type_: str | None = None, **kwargs: Any) -> DialogButton:
         # Defaults
         default_kwargs: dict[str, Any] = {
             'response': None,
@@ -111,7 +101,7 @@ class HigDialog(Gtk.MessageDialog):
                                    buttons=self._buttons_type,
                                    text=text,
                                    use_markup=use_markup,
-                                   secondary_text=secondary_text,
+                                   secondary_text=secondary_text or '',
                                    secondary_use_markup=secondary_use_markup)
 
         self.connect('response', self.on_response)
@@ -149,7 +139,7 @@ class ConfirmationDialog(Gtk.MessageDialog):
                  sec_text: str,
                  buttons: list[DialogButton],
                  modal: bool = True,
-                 transient_for: Optional[Gtk.Window] = None
+                 transient_for: Gtk.Window | None = None
                  ) -> None:
         if transient_for is None:
             transient_for = app.app.get_active_window()
@@ -215,7 +205,7 @@ class ConfirmationCheckDialog(ConfirmationDialog):
                  check_text: str,
                  buttons: list[DialogButton],
                  modal: bool = True,
-                 transient_for: Optional[Gtk.Window] = None
+                 transient_for: Gtk.Window | None = None
                  ) -> None:
         ConfirmationDialog.__init__(self,
                                     title,
@@ -258,7 +248,7 @@ class PastePreviewDialog(ConfirmationDialog):
                  image: GdkPixbuf.Pixbuf,
                  buttons: list[DialogButton],
                  modal: bool = True,
-                 transient_for: Optional[Gtk.Window] = None
+                 transient_for: Gtk.Window | None = None
                  ) -> None:
         ConfirmationDialog.__init__(self,
                                     title,
@@ -294,9 +284,9 @@ class InputDialog(ConfirmationDialog):
                  text: str,
                  sec_text: str,
                  buttons: list[DialogButton],
-                 input_str: Optional[str] = None,
+                 input_str: str | None = None,
                  modal: bool = True,
-                 transient_for: Optional[Gtk.Window] = None,
+                 transient_for: Gtk.Window | None = None,
                  ) -> None:
         ConfirmationDialog.__init__(self,
                                     title,
@@ -327,6 +317,48 @@ class InputDialog(ConfirmationDialog):
         super()._on_response(_dialog, response)
 
 
+class QuitDialog(Gtk.ApplicationWindow):
+    def __init__(self) -> None:
+        Gtk.ApplicationWindow.__init__(
+            self,
+            application=app.app,
+            window_position=Gtk.WindowPosition.CENTER,
+            show_menubar=False,
+            type_hint=Gdk.WindowTypeHint.DIALOG,
+            modal=True,
+            transient_for=app.window,
+            title=_('Quit Gajim')
+        )
+
+        self._ui = get_builder('quit_dialog.ui')
+        self._ui.connect_signals(self)
+        self.add(self._ui.box)
+
+        self.connect('key-press-event', self._on_key_press)
+
+        self.show_all()
+
+    def _on_key_press(self, _widget: QuitDialog, event: Gdk.EventKey) -> None:
+        if event.keyval == Gdk.KEY_Escape:
+            self.destroy()
+
+    def _on_button_clicked(self, button: Gtk.Button) -> None:
+        action = button.get_name()
+
+        if self._ui.remember_checkbutton.get_active():
+            app.settings.set('confirm_on_window_delete', False)
+            app.settings.set('action_on_close', action)
+
+        if action == 'minimize':
+            app.window.minimize()
+        elif action == 'hide':
+            app.window.hide()
+        elif action == 'quit':
+            app.window.quit()
+
+        self.destroy()
+
+
 class ShortcutsWindow:
     def __init__(self):
         transient = app.app.get_active_window()
