@@ -22,11 +22,11 @@ from gajim.common.const import AvatarSize
 from gajim.common.const import StyleAttr
 from gajim.common.events import ApplicationEvent
 from gajim.common.events import MUCNicknameChanged
-from gajim.common.helpers import get_uf_affiliation
-from gajim.common.helpers import get_uf_role
 from gajim.common.helpers import jid_is_blocked
 from gajim.common.i18n import p_
 from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.util.user_strings import get_uf_affiliation
+from gajim.common.util.user_strings import get_uf_role
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.menus import get_groupchat_participant_menu
@@ -55,6 +55,7 @@ class Column(IntEnum):
 
 CONTACT_SIGNALS = {
     'user-affiliation-changed',
+    'user-hats-changed',
     'user-avatar-update',
     'user-joined',
     'user-left',
@@ -218,7 +219,8 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         if not model[iter_][Column.IS_CONTACT]:
             return True
 
-        return self._filter_string in model[iter_][Column.TEXT].lower()
+        nickname = model[iter_][Column.TEXT].split('\n')[0]
+        return self._filter_string in nickname.lower()
 
     def _get_group_iter(self, group_name: str) -> Gtk.TreeIter | None:
         try:
@@ -522,6 +524,7 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
         assert self._contact is not None
         self._contact.multi_connect({
             'user-affiliation-changed': self._update_contact,
+            'user-hats-changed': self._update_contact,
             'user-avatar-update': self._on_user_avatar_update,
             'user-joined': self._on_user_joined,
             'user-left': self._on_user_left,
@@ -530,7 +533,9 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             'user-status-show-changed': self._on_user_status_show_changed,
         })
 
-        for participant in self._contact.get_participants():
+        # Convert (copy) participants iterator to list,
+        # since it may change during iteration, see #11970
+        for participant in list(self._contact.get_participants()):
             self._add_contact(participant)
 
         self._enable_sort(True)
@@ -592,6 +597,9 @@ class GroupchatRoster(Gtk.Revealer, EventHelper):
             name += (f'\n<span size="small" style="italic" alpha="70%">'
                      f'{GLib.markup_escape_text(status)}</span>')
 
+        if contact.hats is not None:
+            name += ' <span size="x-large">🎓</span>'
+
         self._store[iter_][Column.TEXT] = name
 
     def draw_contacts(self) -> None:
