@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -28,16 +18,16 @@ from gajim.common import app
 from gajim.common import ged
 from gajim.common.events import AccountConnected
 from gajim.common.events import AccountDisconnected
-from gajim.common.helpers import event_filter
 from gajim.common.helpers import to_user_string
 from gajim.common.i18n import _
+from gajim.common.util.decorators import event_filter
 
-from .assistant import Assistant
-from .assistant import ErrorPage
-from .assistant import Page
-from .assistant import SuccessPage
+from gajim.gtk.assistant import Assistant
+from gajim.gtk.assistant import ErrorPage
+from gajim.gtk.assistant import Page
+from gajim.gtk.assistant import SuccessPage
 
-log = logging.getLogger('gajim.gui.remove_account')
+log = logging.getLogger('gajim.gtk.remove_account')
 
 
 class RemoveAccount(Assistant):
@@ -181,7 +171,7 @@ class RemoveChoice(Page):
         label.set_halign(Gtk.Align.CENTER)
         label.set_justify(Gtk.Justification.CENTER)
 
-        service = app.settings.get_account_setting(account, 'hostname')
+        service = app.get_hostname_from_account(account)
         check_label = Gtk.Label()
         check_label.set_markup(
             _('Do you want to unregister your account on <b>%s</b> as '
